package com.zaojiao.ui

import android.content.Context
import android.graphics.Canvas
import android.graphics.Path
import android.graphics.RectF
import android.util.AttributeSet
import android.view.ViewGroup

class ClipRRectView @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null,
    defStyle: Int = 0,
) : ViewGroup(context, attrs, defStyle) {
    private val path = Path()
    private var radius = 0f

    init {
        val ta = context.obtainStyledAttributes(attrs, R.styleable.ClipRRectView)
        radius = ta.getDimension(R.styleable.ClipRRectView_radius, 0f)
        ta.recycle()
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        assert(childCount <= 1)

        val child = getChildAt(0)
        measureChild(child, widthMeasureSpec, heightMeasureSpec)
        setMeasuredDimension(child.measuredWidth, child.measuredHeight)
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        val child = getChildAt(0)
        child.layout(l, t, r, b)
    }

    override fun dispatchDraw(canvas: Canvas?) {
        path.reset()
        path.addRoundRect(
            RectF(left.toFloat(), top.toFloat(), right.toFloat(), bottom.toFloat()),
            radius,
            radius,
            Path.Direction.CW
        )

        canvas?.apply {
//            save()
            clipPath(path)
            super.dispatchDraw(canvas)
//            restore()
        }
    }
}